/************************************************************************************
  AddrDisp_r01.ino   LN Version
  Sample Program
/************************************************************************************* 
   This sketch requires the following libraries:
    Loconet Library 1.1.4 

   2021/10/20 Rev01
   2022/02/23 Rev02
   Copyright (c) 2021 Nagoden  All right reserved.
*************************************************************************************/
#include <EEPROM.h>

// include the library code:
// https://github.com/Seeed-Studio/Grove_4Digital_Display
#include "TM1637.h"
//----------------------------------------------------------------------
// Settings used in Loconet
#include <LocoNet.h>
#define DEBUG
#define THROWN 0            //Switch Dir Parameter
#define CLOSED 32           //Switch Dir Parameter
#define DAT_PIN0 A0         //output Sw PORT 1
#define DAT_PIN1 A1         //output Sw PORT 2
#define DAT_PIN2 A2         //output Sw PORT 3
#define DAT_PIN3 A3         //output Sw PORT 4
#define DAT_PIN4 3          //output Sw PORT 5
#define DAT_PIN5 4          //output Sw PORT 6
#define DAT_PIN6 5          //output Sw PORT 7
#define DAT_PIN7 6          //output Sw PORT 8
#define SETUP_PIN 9         //input SetupPin
#define STAT_LED_PIN 10      // Status_LED on/off pin
//Address Disp const
#define BOARDID 3         //BoardID
#define SECTIONID 1         //SectionID
#define ZONEID 1            //ZONE  A:1 B:3 C:5 D:7 E:9 F:11 G:13 H:15

/*********************************************************************************
 Variable definition Address Disp
 ********************************************************************************/
uint16_t  gBoard = 0;
uint16_t  gLocoAddress = 0;
int       gZone  = 0;       
int       gPresent = 0;
//sensor 
uint16_t  gSenBoard = 0;
uint16_t  gSenSection= 0;
uint8_t   gSenPresent = 0;


// Pins definitions for TM1637
const int CLK = A5;     //PC6
const int DIO = A4;     //PC5
// TM1637 and can be changed to other ports
TM1637 tm1637(CLK, DIO);

/*********************************************************************************
 Loconet Class定義
 ********************************************************************************/
LocoNetClass  Loconet;
lnMsg       *LnPacket;
LnBuf       LnTxBuffer;

/*********************************************************************************
 SETUP
 ********************************************************************************/
void setup() {
   Serial.begin(115200);
#ifdef DEBUG   
   Serial.println("Start");
#endif
  /*First initialize the LocoNet interface*/ 
      LocoNet.init(7);

  //LED初期化
      pinMode(DAT_PIN0 , OUTPUT);  // OUTPUTPin1
      //pinMode(DAT_PIN1 , OUTPUT);  // OUTPUTPin2
      //pinMode(DAT_PIN2 , OUTPUT);  // OUTPUTPin3
      //pinMode(DAT_PIN3 , OUTPUT);  // OUTPUTPin4
      //pinMode(DAT_PIN4 , OUTPUT);  // OUTPUTPin5
      //pinMode(DAT_PIN5 , OUTPUT);  // OUTPUTPin6
      //pinMode(DAT_PIN6 , OUTPUT);  // OUTPUTPin7
      //pinMode(DAT_PIN7 , OUTPUT);  // OUTPUTPin8
      //pinMode(SETUP_PIN , INPUT_PULLUP);  // Setup Pin
      pinMode(STAT_LED_PIN , OUTPUT);  // Power LED
      digitalWrite(STAT_LED_PIN, HIGH);
 
  //TML637初期化
  tm1637.init();
  tm1637.set(BRIGHT_TYPICAL); //BRIGHT_TYPICAL = 2,BRIGHT_DARKEST = 0,BRIGHTEST = 7;
  tm1637.displayStr("----"); 
  
}
 

/*********************************************************************************
  Loop処理
 ********************************************************************************/

void loop() {
 /*********************************************
  * Loconet受信処理
  *********************************************/
  // Check for any received LocoNet packets
  LnPacket = LocoNet.receive() ;
  if (LnPacket)  {
      //Serial.println("LnPacket");
      if (!LocoNet.processSwitchSensorMessage(LnPacket)){
        digitalWrite(STAT_LED_PIN, LOW);
        delay(10);
        digitalWrite(STAT_LED_PIN, HIGH);
      }
  }

//アドレス表示
  if  (( gPresent == 1 ) && (BOARDID == gBoard) && (ZONEID == gZone)){
      //LEDに数字表示
      // Without specifying decimal pointt it displays int
      tm1637.displayNum((float)gLocoAddress);
  }
  if  (( gPresent == 0 ) && (BOARDID == gBoard) && (ZONEID == gZone)){
  
      //LEDに数字表示リセット
      tm1637.displayStr("----");
  }
  //変数クリア
  gPresent = 0;
  gBoard = 0 ;
  gZone = 0;
//在線LED出力

  if  (( gSenPresent == 16 ) && (BOARDID == gSenBoard) && (SECTIONID == gSenSection)){
      //LEDに表示
      digitalWrite(DAT_PIN0, HIGH);
#ifdef DEBUG      
      Serial.println("DAT_PIN0:On ");
#endif 
  }
  if  (( gSenPresent == 0 ) && (BOARDID == gSenBoard) && (SECTIONID == gSenSection)){
      //LEDに表示リセット
      digitalWrite(DAT_PIN0, LOW);
#ifdef DEBUG      
      Serial.println("DAT_PIN0:OFF ");
#endif 

  }
 //変数クリア
  gSenPresent = 0;
  gSenBoard = 0 ;
  gSenSection = 0;


}

//以下はLocoNetでのコールバック関数
//--------------------------------------------------------------------------------------------------
//Notifications for accessory control requests
//--------------------------------------------------------------------------------------------------
void notifySwitchRequest( uint16_t Address, uint8_t outData, uint8_t dirData) {
#ifdef DEBUG  
  Serial.print("notifySwitchRequest():");
  Serial.print(Address,DEC);
  Serial.print(",");
  Serial.print(outData,DEC); 
  Serial.print(",");
  Serial.print(dirData,DEC); // 0:t:分岐  32:c:直線
  Serial.println("");
#endif 
}
//--------------------------------------------------------------------------------------------------
// Transponding and Multi-Sense Device info Call-back functions
//--------------------------------------------------------------------------------------------------
void notifyMultiSenseTransponder( uint16_t Address, uint8_t Zone, uint16_t LocoAddress, uint8_t Present ){
    gBoard = ((Address / 16) + 1) ;
    gPresent = Present;
    gZone = Zone;
    gLocoAddress = LocoAddress;
    int section = (Address - (( Address / 16) * 16));
#ifdef DEBUG   
    Serial.print("notifyMultiSenseTransponder: ");
    Serial.print("BoardID: ");
    Serial.print(gBoard);
    Serial.print(", Section: ");
    Serial.print(section );
    Serial.print(", Zone:");   
    Serial.print(Zone); 
    Serial.print(", LocoAddress:");
    Serial.print(LocoAddress,DEC); 
    Serial.print(", Present:");
    Serial.print(Present,DEC); 
    Serial.println("");
#endif   
}
// This call-back function is called from LocoNet.processSwitchSensorMessage
// for all Sensor messages
void notifySensor( uint16_t Address, uint8_t State ) {

gSenBoard = (( Address & 65520) / 16) + 1;  //上位の12ビットがボードのアドレス
gSenSection  = Address & 15;                   //下位の4ビットがSection
gSenPresent = State;                        //在線有無
  
#ifdef DEBUG   
  Serial.print("Sensor: ");
  Serial.print(Address, HEX);
  Serial.print(" ");
  Serial.print(Address, BIN);
  Serial.print(" - ");
  Serial.print( State ? "Active" : "Inactive" );
  //Stateは16で"Active"　0で"Inactive"
  Serial.print(" Address:");
  Serial.print(gSenBoard, DEC);
  Serial.print(" ");
  Serial.print("Section:");
  Serial.println(gSenSection, DEC);
#endif 

}
